@@ -16,10 +16,10 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  ********/
 
-static char rcsid[] = "$Id: disable-paste.c,v 1.2 1998/09/29 20:23:14 itz Exp $";
+static char rcsid[] = "$Id: disable-paste.c,v 1.2 2000/01/17 22:23:03 rubini Exp $";
 
 #include <unistd.h>             /* execv */
 
