@@ -20,7 +20,7 @@
 #line 27 "gpm-root.y"
 
 
-static char rcsid[] = "$Id: gpm-root.y,v 1.15 1999/01/08 19:20:52 itz Exp $";
+static char rcsid[] = "$Id: gpm-root.y,v 1.2 1999/10/07 18:15:18 rubini Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,6 +35,7 @@ static char rcsid[] = "$Id: gpm-root.y,v 1.15 1999/01/08 19:20:52 itz Exp $";
 #include <sys/syslog.h>
 #include <signal.h>         /* sigaction() */
 #include <pwd.h>            /* pwd entries */
+#include <grp.h>            /* initgroups() */
 #include <sys/kd.h>         /* KDGETMODE */
 #include <sys/stat.h>       /* fstat() */
 #include <sys/utsname.h>    /* uname() */
@@ -181,7 +182,7 @@ int f_time(int mode, DrawItem *self, int uid);
 int f_pipe(int mode, DrawItem *self, int uid);
 
 
-#line 191 "gpm-root.y"
+#line 192 "gpm-root.y"
 typedef union {
     int silly;
     char *string;
@@ -256,9 +257,9 @@ static const short yyrhs[] = {    -1,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-   210,   211,   214,   215,   216,   219,   220,   223,   223,   225,
-   226,   227,   228,   229,   232,   232,   234,   236,   237,   240,
-   241,   242
+   211,   212,   215,   216,   217,   220,   221,   224,   224,   226,
+   227,   228,   229,   230,   233,   233,   235,   237,   238,   241,
+   242,   243
 };
 #endif
 
@@ -322,7 +323,7 @@ static const short yycheck[] = {     0,
      3,     9,     9,     9,     3,     0,    -1,    11,    30
 };
 /* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */
-#line 3 "/usr/share/bison.simple"
+#line 3 "/usr/share/misc/bison.simple"
 
 /* Skeleton output parser for bison,
    Copyright (C) 1984, 1989, 1990 Free Software Foundation, Inc.
@@ -339,7 +340,7 @@ static const short yycheck[] = {     0,
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* As a special exception, when this file is copied by Bison into a
    Bison output file, you may use that output file without restriction.
@@ -473,8 +474,10 @@ int yydebug;			/*  nonzero means print parse trace	*/
 
 /* Prevent warning if -Wstrict-prototypes.  */
 #ifdef __GNUC__
+#ifndef YYPARSE_PARAM
 int yyparse (void);
 #endif
+#endif
 
 #if __GNUC__ > 1		/* GNU C and GNU C++ define this.  */
 #define __yy_memcpy(TO,FROM,COUNT)	__builtin_memcpy(TO,FROM,COUNT)
@@ -515,7 +518,7 @@ __yy_memcpy (char *to, char *from, int count)
 #endif
 #endif
 
-#line 196 "/usr/share/bison.simple"
+#line 196 "/usr/share/misc/bison.simple"
 
 /* The user can define YYPARSE_PARAM as the name of an argument to be passed
    into yyparse.  The argument should have type void *.
@@ -820,88 +823,88 @@ yyreduce:
   switch (yyn) {
 
 case 1:
-#line 210 "gpm-root.y"
+#line 211 "gpm-root.y"
 {yyval.draw=cfgall=NULL;;
     break;}
 case 2:
-#line 211 "gpm-root.y"
+#line 212 "gpm-root.y"
 {yyvsp[0].draw->buttons=yyvsp[-1].silly; yyvsp[0].draw->next=yyvsp[-2].draw; yyval.draw=cfgall=yyvsp[0].draw;;
     break;}
 case 3:
-#line 214 "gpm-root.y"
+#line 215 "gpm-root.y"
 {yyval.silly=GPM_B_LEFT;;
     break;}
 case 4:
-#line 215 "gpm-root.y"
+#line 216 "gpm-root.y"
 {yyval.silly=GPM_B_MIDDLE;;
     break;}
 case 5:
-#line 216 "gpm-root.y"
+#line 217 "gpm-root.y"
 {yyval.silly=GPM_B_RIGHT;;
     break;}
 case 6:
-#line 219 "gpm-root.y"
+#line 220 "gpm-root.y"
 {yyval.draw=cfgcurrent=cfg_alloc();;
     break;}
 case 7:
-#line 220 "gpm-root.y"
+#line 221 "gpm-root.y"
 {yyval.draw=yyvsp[-3].draw; yyval.draw->menu=yyvsp[-1].item;;
     break;}
 case 10:
-#line 225 "gpm-root.y"
+#line 226 "gpm-root.y"
 {cfgcurrent->title=yyvsp[0].string;;
     break;}
 case 11:
-#line 226 "gpm-root.y"
+#line 227 "gpm-root.y"
 {cfgcurrent->back=yyvsp[0].silly;;
     break;}
 case 12:
-#line 227 "gpm-root.y"
+#line 228 "gpm-root.y"
 {cfgcurrent->fore=yyvsp[0].silly|yyvsp[-1].silly;;
     break;}
 case 13:
-#line 228 "gpm-root.y"
+#line 229 "gpm-root.y"
 {cfgcurrent->bord=yyvsp[0].silly|yyvsp[-1].silly;;
     break;}
 case 14:
-#line 229 "gpm-root.y"
+#line 230 "gpm-root.y"
 {cfgcurrent->head=yyvsp[0].silly|yyvsp[-1].silly;;
     break;}
 case 15:
-#line 232 "gpm-root.y"
+#line 233 "gpm-root.y"
 {yyval.silly=0;;
     break;}
 case 16:
-#line 232 "gpm-root.y"
+#line 233 "gpm-root.y"
 {yyval.silly=8;;
     break;}
 case 17:
-#line 234 "gpm-root.y"
+#line 235 "gpm-root.y"
 {yyval.item=cfg_cat(yyvsp[-1].item,yyvsp[0].item);;
     break;}
 case 18:
-#line 236 "gpm-root.y"
+#line 237 "gpm-root.y"
 {yyval.item=NULL;;
     break;}
 case 19:
-#line 237 "gpm-root.y"
+#line 238 "gpm-root.y"
 {yyval.item= yyvsp[-1].item ? cfg_cat(yyvsp[-1].item,yyvsp[0].item) : yyvsp[0].item;;
     break;}
 case 20:
-#line 240 "gpm-root.y"
+#line 241 "gpm-root.y"
 {yyval.item=cfg_makeitem('F',yyvsp[-1].string,yyvsp[0].fun, NULL);;
     break;}
 case 21:
-#line 241 "gpm-root.y"
+#line 242 "gpm-root.y"
 {yyval.item=cfg_makeitem('2',yyvsp[-2].string,yyvsp[-1].fun, yyvsp[0].string);;
     break;}
 case 22:
-#line 242 "gpm-root.y"
+#line 243 "gpm-root.y"
 {yyval.item=cfg_makeitem('M',yyvsp[-1].string,NULL,yyvsp[0].draw);;
     break;}
 }
    /* the action file gets copied in in place of this dollarsign */
-#line 498 "/usr/share/bison.simple"
+#line 498 "/usr/share/misc/bison.simple"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1097,7 +1100,7 @@ yyerrhandle:
   yystate = yyn;
   goto yynewstate;
 }
-#line 245 "gpm-root.y"
+#line 246 "gpm-root.y"
  /* end grammar ###########################################################*/
 
 int yyerror(char *s){
@@ -1303,6 +1306,7 @@ DrawItem *cfg_cat(DrawItem *d1, DrawItem *d2)
 void f__fix(struct passwd *pass)
 {
   setgid(pass->pw_gid);
+  initgroups(pass->pw_name, pass->pw_gid);
   setuid(pass->pw_uid);
   setenv("HOME",    pass->pw_dir, 1);
   setenv("LOGNAME", pass->pw_name,1);
@@ -1395,10 +1399,9 @@ int f_bgcmd(int mode, DrawItem *self, int uid)
           return 1;
 
 	case 0:
-	  setuid(uid);
 	  pass=getpwuid(uid);
 	  if (!pass) exit(1);
-	  f__fix(pass);
+	  f__fix(pass); /* setgid(), setuid(), setenv(), ... */
 	  close(0); close(1); close(2);
 	  open("/dev/null",O_RDONLY); /* stdin  */
 	  open("/dev/tty0",O_WRONLY); /* stdout */
@@ -1684,7 +1687,7 @@ Draw *cfg_read(int uid)
 
 
 /*---------------------------------------------------------------------*/
-/* the return value tells wether it has been newly loaded or not */
+/* the return value tells whether it has been newly loaded or not */
 int getdraw(int uid, int buttons, time_t mtime1, time_t mtime2)
 {
   struct passwd *pass;
@@ -1711,7 +1714,7 @@ int getdraw(int uid, int buttons, time_t mtime1, time_t mtime2)
 
   if (stat(cfgname,&buf)==-1)
     {
-      gpm_debug_log(LOG_ERR,"stat (%s) failed",cfgname);
+      gpm_debug_log(LOG_DEBUG,"stat (%s) failed",cfgname);
       /* try the system wide */
       mtime=mtime2; uid = -1;
       strcpy(cfgname,SYSTEM_CFG);
