@@ -1,9 +1,9 @@
 /*
  * general purpose mouse support for Linux
  *
- * Copyright 1993        ajh@gec-mrc.co.uk (Andrew Haylett)
- * Copyright 1994-1997   rubini@linux.it (Alessandro Rubini)
- * Copyright (C) 1998	Ian Zimmerman <itz@rahul.net>
+ * Copyright (C) 1993        Andreq Haylett <ajh@gec-mrc.co.uk>
+ * Copyright (C) 1994-1999   Alessandro Rubini <rubini@linux.it>
+ * Copyright (C) 1998 	     Ian Zimmerman <itz@rahul.net>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -17,10 +17,10 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  ********/
 
-static char rcsid[] = "$Id: gpm.c,v 1.15 1999/02/19 05:51:49 itz Exp $";
+static char rcsid[] = "$Id: gpm.c,v 1.6 2000/02/07 09:45:05 rubini Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -60,7 +60,8 @@ struct mouse_features mouse_table[3] = {
   {
   DEF_TYPE, DEF_DEV, DEF_SEQUENCE,
   DEF_BAUD, DEF_SAMPLE, DEF_DELTA, DEF_ACCEL, DEF_SCALE, 0 /* scaley */,
-  DEF_TIME, DEF_CLUSTER, DEF_THREE, DEF_TOGGLE, DEF_GLIDEPOINT_TAP,
+  DEF_TIME, DEF_CLUSTER, DEF_THREE, DEF_GLIDEPOINT_TAP,
+  (char *)NULL /* extra */,
   (Gpm_Type *)NULL
   },
 };
@@ -94,11 +95,50 @@ fd_set selSet, readySet, connSet;
 time_t last_selection_time;
 time_t opt_age_limit = 0;
 
+/* argc and argv for mice initialization */
+static int mouse_argc[3]; /* 0 for default (unused) and two mice */
+static char **mouse_argv[3]; /* 0 for default (unused) and two mice */
+
 /*===================================================================*/
 /*
- *      first, all the stuff that used to be in gpn.c 
+ *      first, all the stuff that used to be in gpn.c (i.e., not main-loop)
  */
 /*-------------------------------------------------------------------*/
+
+/* build_argv is used for mouse initialization routines */
+static char **build_argv(char *argv0, char *str, int *argcptr, char sep)
+{
+    int argc = 1;
+    char **argv;
+    char *buffer, *s;
+
+    /* argv0 is never NULL, but the extra string may well be */
+    if (str)
+        for (s=str; sep && (s = strchr(s, sep)); argc++)
+	    s++;
+    argv = calloc(argc+2, sizeof(char **));
+    if (!argv)
+        oops("allocation failed\n");
+    argv[0] = argv0;
+
+    if (!str) {
+        *argcptr = argc; /* 1 */
+	return argv;
+    }
+    /* else, add arguments */
+    s = argv[1] = strdup(str);
+    argc = 2; /* first to fill */
+
+    /* ok, now split: the first one is in place, and s is the whole string */
+    for ( ; sep && (s = strchr(s, sep)) ; argc++) {
+	*s = '\0';
+	s++;
+	argv[argc] = s;
+    }
+    *argcptr = argc;
+    return argv;
+}
+
 /*-------------------------------------------------------------------*/
 static inline int open_console(const int mode)
 {
@@ -126,21 +166,12 @@ static inline int wait_text(int *fdptr)
     }
   while (kd_mode != KD_TEXT);
 
-  /* reopen, reinit */
+  /* reopen, reinit (the function is only used if we have one mouse device) */
   if ((*fdptr=open(opt_dev,O_RDWR))<0)
     oops(opt_dev);
   if (m_type->init)
-    m_type=(m_type->init)(*fdptr, m_type->flags, m_type);
-
-  if (opt_toggle)
-    {
-    unsigned int modem_lines;
-
-    ioctl(fd, TIOCMGET, &modem_lines);
-    modem_lines &= ~opt_toggle;
-    ioctl(fd, TIOCMSET, &modem_lines);
-    }
-
+    m_type=(m_type->init)(*fdptr, m_type->flags, m_type,
+			  mouse_argc[1], mouse_argv[1]);
   return (1);
 }
 
@@ -317,7 +348,7 @@ static inline char *getMouseData(int fd, Gpm_Type *type, int kd_mode)
         gpm_debug_log(LOG_DEBUG,"Extra %02x",data[0]);
 	return data;
 	}
-      gpm_debug_log(LOG_ERR,"Error in protocol");
+      gpm_debug_log(LOG_DEBUG,"Error in protocol");
       return NULL;
       }
 
@@ -356,6 +387,7 @@ static inline char *getMouseData(int fd, Gpm_Type *type, int kd_mode)
 
 static int statusX,statusY,statusB; /* to return info */
 static int statusC=0; /* clicks */
+void get_console_size(Gpm_Event *ePtr);
 
 /*-------------------------------------------------------------------*/
 static inline int processMouse(int fd, Gpm_Event *event, Gpm_Type *type,
@@ -501,13 +533,26 @@ static inline int processMouse(int fd, Gpm_Event *event, Gpm_Type *type,
   statusB=event->buttons;
 
   i=open_console(O_RDONLY);
-  ioctl(i,VT_GETSTATE,&stat);
+  /* modifiers */
   j = event->modifiers; /* save them */
   event->modifiers=6; /* code for the ioctl */
   if (ioctl(i,TIOCLINUX,&(event->modifiers))<0)
     oops("get_shift_state");
-  close(i);
   event->modifiers |= j; /* add mouse-specific bits */
+
+  /* status */
+  j = stat.v_active;
+  if (ioctl(i,VT_GETSTATE,&stat)<0)
+    oops("get_console_status");
+
+  /*
+   * if we changed console, request the current console size,
+   * as different consoles can be of different size
+   */
+  if (stat.v_active != j)
+    get_console_size(event);
+  close(i);
+
   event->vc = stat.v_active;
 
   if (oldB==event->buttons)
@@ -606,7 +651,7 @@ static int get_data(Gpm_Connect *where, int whence)
 
   if ((i=read(whence, where, sizeof(Gpm_Connect)))!=sizeof(Gpm_Connect))
     {
-    gpm_debug_log(LOG_NOTICE, "No data");
+    /* gpm_debug_log(LOG_NOTICE, "No data"); -- don't log, we are closing */
     return i ? -1 : 0;
     }
 
@@ -828,9 +873,12 @@ static inline int processConn(int fd) /* returns newfd or -1 */
 /*-------------------------------------------------------------------*/
 void get_console_size(Gpm_Event *ePtr)
 {
-  int i;
+  int i, prevmaxx, prevmaxy;
   struct mouse_features *which_mouse; /* local */
 
+  /* before asking the new console size, save the previous values */
+  prevmaxx = maxx; prevmaxy = maxy;
+
   i=open_console(O_RDONLY);
   ioctl(i, TIOCGWINSZ, &win);
   close(i);
@@ -841,7 +889,14 @@ void get_console_size(Gpm_Event *ePtr)
     }
   maxx=win.ws_col; maxy=win.ws_row;
   gpm_debug_log(LOG_DEBUG,"%i - %i",maxx,maxy);
-  statusX=ePtr->x=maxx/2; statusY=ePtr->y=maxy/2;
+
+  if (!prevmaxx) { /* first invocation, place the pointer in the middle */
+      statusX = ePtr->x = maxx/2;
+      statusY = ePtr->y = maxy/2;
+  } else { /* keep the pointer in the same position where it was */
+      statusX = ePtr->x = ePtr->x * maxx / prevmaxx;
+      statusY = ePtr->y = ePtr->y * maxy / prevmaxy;
+  } 
 
   for (i=1; i <= 1+opt_double; i++)
     {
@@ -906,25 +961,22 @@ int main(int argc, char **argv)
       /* and then reset the flag */
       fcntl(fd,F_SETFL,fcntl(fd,F_GETFL) & ~O_NDELAY);
 
-      /* init the device, and change mouse type */
+      /* create argc and argv for this device */
+      mouse_argv[i] = build_argv(opt_type, opt_options, &mouse_argc[i], ',');
+
+      /* init the device, and use the return value as new mouse type */
       if (m_type->init)
-        m_type=(m_type->init)(fd, m_type->flags, m_type);
+        m_type=(m_type->init)(fd, m_type->flags, m_type, 
+			      mouse_argc[i], mouse_argv[i]);
       if (!m_type)
         { oops("mouse initialization failed"); }
 
-      if (opt_toggle)
-        {
-          unsigned int modem_lines;
-
-          ioctl(fd, TIOCMGET, &modem_lines);
-          modem_lines &= ~opt_toggle;
-          ioctl(fd, TIOCMSET, &modem_lines);
-        }
-
       which_mouse->fd=fd;
       maxfd=max(fd, maxfd);
     } /*for*/
 
+  /* FIXME: stderr must be closed at this point, as protocol init needs it */
+
 /*....................................... catch interesting signals */
 
   signal(SIGTERM, gpm_killed);
