@@ -16,16 +16,16 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  ********/
 
 #ifndef _GPMCFG_INCLUDED
 #define _GPMCFG_INCLUDED
 
-/* $Id: gpmCfg.h,v 1.6 1999/01/03 21:02:51 itz Exp $ */
+/* $Id: gpmCfg.h,v 1.3 2000/01/17 22:23:03 rubini Exp $ */
 
 #define GPM_NAME    "gpm-Linux"
-#define GPM_DATE    "$Date: 1999/01/03 21:02:51 $"
+#define GPM_DATE    "$Date: 2000/01/17 22:23:03 $"
 
 /* timeout for the select() syscall */
 #define SELECT_TIME 86400 /* one day */
@@ -61,7 +61,6 @@
 
 #define DEF_TEST             0
 #define DEF_PTRDRAG          1    /* double or triple click */
-#define DEF_TOGGLE           0
 #define DEF_GLIDEPOINT_TAP   0    /* tapping emulates no buttons by default */
 
 #endif /* _GPMCFG_INCLUDED */
