@@ -1,9 +1,9 @@
 /*
  * gpn.c - support functions for gpm-Linux
  *
- * Copyright 1993        ajh@gec-mrc.co.uk (Andrew Haylett)
- * Copyright 1994,1995   rubini@linux.it (Alessandro Rubini)
- * Copyright (C) 1998 	Ian Zimmerman <itz@rahul.net>
+ * Copyright (C) 1993        Andreq Haylett <ajh@gec-mrc.co.uk>
+ * Copyright (C) 1994-1999   Alessandro Rubini <rubini@linux.it>
+ * Copyright (C) 1998 	     Ian Zimmerman <itz@rahul.net>
  *
  * Tue,  5 Jan 1999 23:26:10 +0000, modified by James Troup <james@nocrew.org>
  * (usage): typo (s/an unexistent/a non-existent/)
@@ -20,11 +20,9 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  ********/
 
-static char rcsid[] = "$Id: gpn.c,v 1.20 1999/04/06 09:22:01 itz Exp $";
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>        /* strerror(); ?!? memcpy() */
@@ -199,7 +197,7 @@ usage(char *whofailed)
          "    -m mouse-device  sets mouse device\n"
          "    -M               enable multiple mouse. Following options refer to\n"
          "                     the second device. Forces \"-R\"\n"
-         "    -o modem-lines   toggle modem lines (\"dtr\", \"rts\", \"both\")\n"
+         "    -o options       decoder-specific options (e.g.: \"dtr\", \"rts\")\n"
          "    -p               draw the pointer while striking a selection\n"
          "    -q               quit after changing mouse behaviour\n"
          "    -r number        sets the responsiveness (default %i)\n"
@@ -292,13 +290,37 @@ check_uniqueness(void)
   } /*if*/
 }
 
+/* the function returns a valid type pointer or NULL if not found*/
+static struct Gpm_Type *find_mouse_by_name(char *name)
+{
+    Gpm_Type *type;
+    char *s;
+    int len = strlen(name);
+
+    for (type=mice; type->fun; type++) {
+    
+	if (!strcasecmp(name, type->name))
+	    break;
+	/* otherwise, look in the synonym list */
+	for (s = type->synonyms; s; s = strchr(s, ' ')) {
+	    while (*s && isspace(*s))
+	        s++; /* skip spaces */
+	    if (!strncasecmp(name, s, len) && !isprint(*(s + len)))
+	        break; /* found */
+	}
+	if (s) break; /* found a synonym */
+    }
+    if (!type->fun)
+        return NULL;
+    return type;
+}
+
 /*===================================================================*/
 int 
 cmdline(int argc, char **argv)
 {
   char options[]="a:A::b:B:d:Dg:hi:kl:m:Mo:pqr:R::s:S:t:TvV::23";
   int i, opt;
-  FILE *f;
   static struct {char *in; char *out;} seq[] = {
     {"123","01234567"},
     {"132","02134657"},
@@ -345,12 +367,7 @@ cmdline(int argc, char **argv)
           if (0 == opt_repeater_type)
             opt_repeater_type = "msc";
           which_mouse=mouse_table+2; break;
-        case 'o':
-          if (!strcmp(optarg,"dtr"))       opt_toggle=TIOCM_DTR;
-          else if (!strcmp(optarg,"rts"))  opt_toggle=TIOCM_RTS;
-          else if (!strcmp(optarg,"both")) opt_toggle=TIOCM_DTR | TIOCM_RTS;
-          else exit(usage("toggle lines"));
-          break;
+	case 'o': opt_options = optarg; break;
         case 'p': opt_ptrdrag=0; break;
         case 'q': opt_quit++; gpm_log_daemon = 0; break;
         case 'r':
@@ -422,15 +439,10 @@ cmdline(int argc, char **argv)
       if (!seq[opt].in) exit(usage("button sequence"));
       opt_sequence=strdup(seq[opt].out); /* I can rewrite on it */
 
-      /* look for the type */
-      for (m_type=mice; m_type->fun; m_type++)
-        if (!strcmp(opt_type,m_type->name)
-            || !strcasecmp(opt_type,m_type->syn))
-          break;
-
-      if (!(m_type->fun)) /* not found */
+      /* look for the mouse type */
+      m_type = find_mouse_by_name(opt_type);
+      if (!m_type) /* not found */
         exit(M_listTypes());
-
     } /*for*/
 
   if (opt_repeater) {
@@ -438,12 +450,9 @@ cmdline(int argc, char **argv)
       opt_rawrep = 1;
     } else {
       /* look for the type */
-      for (repeated_type=mice; repeated_type->fun; repeated_type++)
-        if (!strcmp(opt_repeater_type,repeated_type->name)
-            || !strcasecmp(opt_repeater_type,repeated_type->syn))
-          break;
+      repeated_type = find_mouse_by_name(opt_repeater_type);
 
-      if (!(repeated_type->fun)) /* not found */
+      if (!repeated_type) /* not found */
         exit(M_listTypes());
 
       if (!(repeated_type->repeat_fun)) /* unsupported translation */
@@ -472,7 +481,7 @@ cmdline(int argc, char **argv)
 
     fclose(stdin); fclose(stdout);
 
-#if (!defined(HAVE_SYSLOG) || !defined(HAVE_VSYSLOG))
+#if 1 /* was: "if (!defined(HAVE_SYSLOG) || !defined(HAVE_VSYSLOG))" */
     if (!freopen("/dev/console","w",stderr)) /* the currently current console */
       {
         oops("freopen(stderr) failed");
@@ -502,7 +511,12 @@ cmdline(int argc, char **argv)
   
 
 
+/*
+ * Use 4 as indent-level. It used to be 2, but new stuff will be inserted
+ * using 4.
+ */
+
 /* Local Variables: */
-/* c-indent-level: 2 */
+/* c-indent-level: 4 */
 /* End: */
 
