@@ -19,16 +19,17 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  ********/
 
-static char rcsid[] = "$Id: liblow.c,v 1.6 1999/04/20 14:16:54 itz Exp $";
+static char rcsid[] = "$Id: liblow.c,v 1.5 2000/01/27 22:09:00 rubini Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <string.h>        /* strncmp */
 #include <unistd.h>        /* select(); */
+#include <errno.h>
 #include <sys/time.h>      /* timeval */
 #include <sys/types.h>     /* socket() */
 #include <sys/socket.h>    /* socket() */
@@ -49,8 +50,6 @@ static char rcsid[] = "$Id: liblow.c,v 1.6 1999/04/20 14:16:54 itz Exp $";
 
 #include "gpmInt.h"
 
-extern int errno;
-
 #ifndef min
 #define min(a,b) ((a)<(b) ? (a) : (b))
 #define max(a,b) ((a)>(b) ? (a) : (b))
@@ -430,7 +429,14 @@ int Gpm_GetEvent(Gpm_Event *event)
           return 0;
         }
 #endif
-      gpm_debug_log(LOG_NOTICE,"Read too few bytes (%i) at %s:%d",count,__FILE__,__LINE__);
+      /*
+       * avoid to send the message if there is no data; sometimes it makes
+       * sense to poll the mouse descriptor any now an then using a
+       * non-blocking descriptor
+       */
+      if (count != -1 || errno != EAGAIN)
+	  gpm_debug_log(LOG_NOTICE,"Read too few bytes (%i) at %s:%d",
+			count,__FILE__,__LINE__);
       return -1;
     }
 
@@ -441,6 +447,16 @@ int Gpm_GetEvent(Gpm_Event *event)
 }
 
 
+#define MAXNBPREVCHAR 4         /* I don't think more is usefull, JD */
+static int nbprevchar=0, prevchar[MAXNBPREVCHAR];
+
+/*-------------------------------------------------------------------*/
+int Gpm_CharsQueued ()
+{
+  return nbprevchar;
+}
+
+
 /*-------------------------------------------------------------------*/
 int Gpm_Getc(FILE *f)
 {
@@ -498,10 +514,8 @@ int Gpm_Getc(FILE *f)
 #define DELAY_MS 100
         static struct timeval to={0,DELAY_MS*1000};
         static fd_set selSet;
-#define MAXNBPREVCHAR 4         /* I don't think more is usefull, JD */
-        static int nbprevchar=0, prevchar[MAXNBPREVCHAR];
-        extern gpm_convert_event(char *data, Gpm_Event *event);
-        int c; char mdata[4];
+        extern gpm_convert_event(unsigned char *data, Gpm_Event *event);
+        int c; unsigned char mdata[4];
 
         if (nbprevchar)  /* if there are some consumed char ... */
           return prevchar[--nbprevchar];
@@ -584,7 +598,7 @@ int Gpm_FitValuesM(int *x, int *y, int margin)
 }
 
 /*-------------------------------------------------------------------*/
-int gpm_convert_event(char *mdata, Gpm_Event *ePtr)
+int gpm_convert_event(unsigned char *mdata, Gpm_Event *ePtr)
 {
   static struct timeval tv1={0,0}, tv2;
   static int clicks=0;
