@@ -16,10 +16,10 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  ********/
 
-static char rcsid[] = "$Id: mev.c,v 1.5 1999/03/31 07:59:38 itz Exp $";
+static char rcsid[] = "$Id: mev.c,v 1.4 2000/01/17 22:23:03 rubini Exp $";
 
 /*
  * This client is meant to be used both interactively to check
@@ -40,6 +40,7 @@ static char rcsid[] = "$Id: mev.c,v 1.5 1999/03/31 07:59:38 itz Exp $";
 #include <sys/time.h>
 #include <unistd.h>
 #include <termios.h>
+#include <errno.h>
 
 #include <linux/keyboard.h> /* to use KG_SHIFT and so on */
 
@@ -423,8 +424,6 @@ int interact(char *cmd) /* returns 0 on success and !=0 on error */
 int main(int argc, char **argv)
 {
   Gpm_Connect conn;
-  int c,d;
-  char s[8];
   char cmd[128];
   char* tty;
   Gpm_Handler* my_handler;       /* not the real gpm handler! */
@@ -470,7 +469,8 @@ int main(int argc, char **argv)
       FD_SET(STDIN_FILENO, &readset);
     } /*if*/
 
-    select(gpm_fd+1, &readset, 0, 0, 0);
+    if (select(gpm_fd+1, &readset, 0, 0, 0) < 0 && errno == EINTR)
+	continue;
 
     if (FD_ISSET(STDIN_FILENO, &readset)) {
       if (0 == fgets(cmd, sizeof(cmd), stdin) ||
