@@ -3,7 +3,7 @@
  * mice.c - mouse definitions for gpm-Linux
  *
  * Copyright (C) 1993        Andrew Haylett <ajh@gec-mrc.co.uk>
- * Copyright (C) 1994-1999   Alessandro Rubini <rubini@linux.it>
+ * Copyright (C) 1994-2000   Alessandro Rubini <rubini@linux.it>
  * Copyright (C) 1998,1999   Ian Zimmerman <itz@rahul.net>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -18,11 +18,9 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  ********/
 
-static char rcsid[] = "$Id: mice.c,v 1.15 1999/03/31 08:14:03 itz Exp $";
-
 /*
  * This file is part of the mouse server. The information herein
  * is kept aside from the rest of the server to ease fixing mouse-type
@@ -44,12 +42,15 @@ static char rcsid[] = "$Id: mice.c,v 1.15 1999/03/31 08:14:03 itz Exp $";
  * Refer to the definition of Gpm_Type to probe further.
  */
 
+#include <stdio.h>
+#include <stdlib.h>
 #include <termios.h>
 #include <fcntl.h>
 #include <termios.h>
 #include <errno.h>
 #include <unistd.h>
 #include <strings.h>
+#include <ctype.h>
 
 #include <sys/types.h>
 #include <sys/stat.h> /* stat() */
@@ -67,12 +68,143 @@ static char rcsid[] = "$Id: mice.c,v 1.15 1999/03/31 08:14:03 itz Exp $";
 #include "twiddler.h"
 #include "synaptics.h"
 
-/*  real absolute coordinates for absolute devices: min 0 max=16383 */
+/*========================================================================*/
+/* Parsing argv: helper dats struct function (should they get elsewhere?) */
+
+enum argv_type {
+    ARGV_BOOL = 1,
+    ARGV_INT, /* "%i" */
+    ARGV_DEC, /* "%d" */
+    ARGV_STRING,
+    /* other types must be added */
+    ARGV_END = 0
+};
+
+typedef struct argv_helper {
+    char *name;
+    enum argv_type type;
+    union u {
+	int *iptr;   /* used for int and bool arguments */
+	char **sptr; /* used for string arguments, by strdup()ing the value */
+    } u;
+    int value; /* used for boolean arguments */
+} argv_helper;
+
+static int parse_argv(argv_helper *info, int argc, char **argv)
+{
+    int i, j, errors = 0;
+    long l;
+    argv_helper *p;
+    char *s, *t;
+    int base = 0; /* for strtol */
+
+
+    for (i=1; i<argc; i++) {
+	for (p = info; p->type != ARGV_END; p++) {
+	    j = strlen(p->name);
+	    if (strncmp(p->name, argv[i], j))
+		continue;
+	    if (isalnum(argv[i][j]))
+		continue;
+	    break;
+	}
+	if (p->type == ARGV_END) { /* not found */
+	    fprintf(stderr, "%s: Uknown option \"%s\" for pointer \"%s\"\n",
+		    prgname, argv[i], argv[0]);
+	    errors++;
+	    continue;
+	}
+	/* Found. Look for trailing stuff, if any */
+	s = argv[i]+j;
+	while (*s && isspace(*s)) s++; /* skip spaces */
+	if (*s == '=') s++; /* skip equal */
+	while (*s && isspace(*s)) s++; /* skip other spaces */
+
+	/* Now parse what s is */
+	switch(p->type) {
+	    case ARGV_BOOL:
+		if (*s) {
+		    fprintf(stderr, "%s: Option \"%s\" takes no argument: "
+			    "ignoring \"%s\"\n", prgname, p->name, s);
+		    errors++;
+		}
+		*(p->u.iptr) = p->value;
+		break;
+
+	    case ARGV_DEC:
+		base = 10; /* and fall through */
+	    case ARGV_INT:
+		l = strtol(s, &t, base);
+		if (*t) {
+		    fprintf(stderr, "%s: Invalid arg. \"%s\" to \"%s\"\n",
+			    prgname, s, p->name);
+		    errors++;
+		    break;
+		}
+		*(p->u.iptr) = (int)l;
+		break;
+
+	    case ARGV_STRING:
+		*(p->u.sptr) = strdup(s);
+		break;
+
+	    case ARGV_END: /* let's please "-Wall" */
+		break;
+	}
+    } /* for i in argc */
+    if (errors)
+	fprintf(stderr, "%s: Continuing despite errors in option parsing\n",
+		prgname);
+    return errors;
+}
+
+/* Provide a common error engine by parsing with an empty option-set */
+static volatile int check_no_argv(int argc, char **argv)
+{
+    static argv_helper optioninfo[] = {
+	{"",       ARGV_END}
+    };
+    return parse_argv(optioninfo, argc, argv);
+}
+
+/* Parse the "old" -o options */
+static int option_modem_lines(int fd, int argc, char **argv)
+{
+    static unsigned int err, lines, reallines;
+
+    static argv_helper optioninfo[] = {
+	{"dtr",  ARGV_BOOL, u: {iptr: &lines}, value: TIOCM_DTR},
+	{"rts",  ARGV_BOOL, u: {iptr: &lines}, value: TIOCM_RTS},
+	{"both", ARGV_BOOL, u: {iptr: &lines}, value: TIOCM_DTR | TIOCM_RTS},
+	{"",       ARGV_END}
+    };
+
+    if (argc<2) return 0;
+    if (argc > 2) {
+	fprintf(stderr, "%s: Too many options for \"-t %s\"\n",
+		prgname, argv[0]);
+	errno = EINVAL; /* used by gpm_oops(), if the caller reports failure */
+	return -1;
+    }
+    err = parse_argv(optioninfo, argc, argv);
+    if (err) return 0; /* a message has been printed, but go on as good */
+
+    /* ok, move the lines */
+    ioctl(fd, TIOCMGET, &reallines);
+    reallines &= ~lines;
+    ioctl(fd, TIOCMSET, &reallines);
+    return 0;
+}
+
+/*========================================================================*/
+/*  real absolute coordinates for absolute devices,  not very clean */
+
+#define REALPOS_MAX 16383 /*  min 0 max=16383, but due to change.  */
 int realposx=-1, realposy=-1;
 
 /*========================================================================*/
 /*
- * Ok, here we are: first, provide the functions.
+ * Ok, here we are: first, provide the functions; initialization is later.
  * The return value is the number of unprocessed bytes
  */
 
@@ -163,7 +295,7 @@ static int M_summa(Gpm_Event *state, unsigned char *data)
 
   y = ((data[4]<<7) | data[3])-SUMMA_BORDER; if (y<0) y=0; if (y>summamaxy) y=summamaxy;
   state->y = 1 + y * (win.ws_row-1)/summamaxy;
-  realposy = 1 + y * 16383 / summamaxy;  
+  realposy = y * 16383 / summamaxy;  
 
   state->buttons=
     !!(data[0]&1) * GPM_B_LEFT +
@@ -404,10 +536,13 @@ static int M_netmouse(Gpm_Event *state,  unsigned char *data)
   return 0;
 }
 
-static Gpm_Type *I_netmouse(int fd, unsigned short flags, struct Gpm_Type *type)
+static Gpm_Type *I_netmouse(int fd, unsigned short flags,
+			    struct Gpm_Type *type, int argc, char **argv)
 {
   unsigned char magic[6] = { 0xe8, 0x03, 0xe6, 0xe6, 0xe6, 0xe9 };
   int i;
+
+  if (check_no_argv(argc, argv)) return NULL;
   for (i=0; i<6; i++) {
     unsigned char c = 0;
     write( fd, magic+i, 1 );
@@ -446,7 +581,6 @@ static int M_mman(Gpm_Event *state,  unsigned char *data)
   if (data[1]==GPM_EXTRA_MAGIC_1 && data[2]==GPM_EXTRA_MAGIC_2)
     {
     /* got unexpected fourth byte */
-    gpm_debug_log(LOG_NOTICE,"Extra byte = %02x",*data);
     if ((b=(*data>>4)) > 0x3) return -1;  /* just a sanity check */
     state->dx=state->dy=0;
     
@@ -488,37 +622,124 @@ static int M_mman(Gpm_Event *state,  unsigned char *data)
   return 0;
 }
 
-/* wacom tablet, in native mode */
+/* 
+ *  Wacom Tablets with pen and mouse:
+ *  Relative-Mode And Absolute-Mode; 
+ *  Stefan Runkel  01/2000 <runkel@runkeledv.de>,
+ *  Mike Pioskowik 01/2000 <pio.d1mp@nbnet.de>
+*/
+/* for relative and absolute : */
+int WacomModell =-1;          /* -1 means "dont know" */
+int WacomAbsoluteWanted=0;    /* Tell Driver if Relative or Absolute */
+int wmaxx, wmaxy;         
+char upmbuf[25]; /* needed only for macro buttons of ultrapad */
+
+/* Data for Wacom Modell Identification */
+/* (MaxX, MaxY are for Modells which do not answer resolution requests */
+struct WC_MODELL{
+  char name[15];       
+  char magic[3];
+  int  maxX;
+  int  maxY;
+  int  border;
+  int  treshold;
+} wcmodell[] = {
+  /* ModellName    Magic     MaxX     MaxY  Border  Tresh */
+     "UltraPad"  , "UD",        0,       0,    250,    20, 
+  /* "Intuos"    , "GD",        0,       0,      0,    20, not yet supported */
+     "PenPartner", "CT",        0,       0,      0,    20, 
+     "Graphire"  , "ET",     5103,    3711,      0,    20
+  };
+
+#define IsA(m) ((WacomModell==(-1))? 0:!strcmp(#m,wcmodell[WacomModell].name))
+
 static int M_wacom(Gpm_Event *state, unsigned char *data)
 {
-  static int ox=-1, oy, threshold=8;
-  int x, y, z;
+  static int ox=-1, oy;
+  int x, y;
+  int macro=0;      /* macro buttons from tablet */
 
-  if (data[0]==0xa0) /* leave -- but sometimes it's not decoded */
-    { ox=-1; state->buttons=0; state->dx=state->dy=0; return 0;}
+  /* Bit [0]&64 seems to have different meanings -
+   *   graphire:   inside of active area;
+   *   Ultrapad:   Pen in proximity (means: pen detected)
+   *   Penpartner: no idea...
+   * this may be of worth sometimes, but for now, we can say, that
+   * if the graphire tells us that we are in the active area, we can tell
+   * also that the graphire has the pen in proximity.
+   */
 
-  
-  x = (data[1]<<7) | (data[2]&0x7f);
-  y = (data[4]<<7) | (data[5]&0x7f);
-  z = data[6]-64 + 128*(data[6]<64);
-  if (ox==-1 || abs(x-ox)>18 || abs(y-oy)>18) { ox=x; oy=y;} /* enter */
+  if (!(data[0]&64)) /* Tool not in proximity or out of active area */ 
+     {
+	/* handle Ultrapad macro buttons, if we get a packet without
+	 * proximity bit but with the buttonflag set, we know that we have
+	 * a macro event */
+	if (IsA(UltraPad)) 
+	  {
+	     if (data[0]&8) /* here: macro button has been pressed */
+	       {
+                 if (data[3]&8)  macro=(data[6]); 
+                 if (data[3]&16) macro=(data[6])+12;
+	         if (data[3]&32) macro=(data[6])+24;  /* rom-version >= 1.3 */
 
-  state->buttons = (GPM_B_MIDDLE * !!(data[3]&0x10)); /* the b. means middle */
-  if (!state->buttons)
-    if (data[3]&0x20) state->buttons = (z>threshold) * GPM_B_RIGHT;
-    else              state->buttons = (z>threshold) * GPM_B_LEFT;
+                 state->modifiers=macro;
+                 /* Here we simulate the middle mousebutton */
+                 /* with ultrapad Eprom Version 1.2 */
+                 /*
+	         gpm_debug_log(LOG_NOTICE, "WACOM Macro: %d \n", macro); 
+	         if (macro==12) state->buttons =  GPM_B_MIDDLE;
+                 */
+	     } /* end if macrobutton pressed */
+          } /* end if ultrapad */
+          
+          if (!IsA(UltraPad)){ /* Tool out of active area */ 
+             ox=-1; 
+             state->buttons=0; 
+             state->dx=state->dy=0; 
+          }
 
-  if (z>threshold) threshold=1;
-  else threshold=8;
+	  return 0; /* nothing more to do so leave */
+    } /* end if Tool out of active area*/ 
 
-  state->dx = (x-ox)/5; 
-  state->dy = (y-oy)/5; /* divide by 5, 'cause it's too much sensitive */
+    x = (((data[0] & 0x3) << 14) +  
+        (data[1] << 7) +
+        data[2]); 
+    y = (((data[3] & 0x3) << 14) +
+        (data[4] << 7) +
+        data[5]);
+   
+    if (WacomAbsoluteWanted){                               /* Absolute Mode */
+       if (x>wmaxx) x=wmaxx; if (x<0) x=0;
+       if (y>wmaxy) y=wmaxy; if (y<0) y=0;
+       state->x  = (x * win.ws_col / wmaxx);
+       state->y  = (y * win.ws_row / wmaxy);
+       
+       realposx = (x / wmaxx); /* this two lines come from the summa driver. */
+       realposy = (y / wmaxy); /* they seem to be buggy (always give zero).  */
 
-  ox=x; oy=y;
-  return 0;
+    }
+    else {                                               /* Relative Mode */
+      /* Treshold; if greather then treat tool as first time in proximity */
+      if (
+             abs(x-ox)>(wmaxx/wcmodell[WacomModell].treshold) 
+          || abs(y-oy)>(wmaxy/wcmodell[WacomModell].treshold) 
+         )
+      {
+         ox=x; oy=y;
+      }
+
+      state->dx= (x-ox) / (wmaxx / win.ws_col / wcmodell[WacomModell].treshold);
+      state->dy= (y-oy) / (wmaxy / win.ws_row / wcmodell[WacomModell].treshold);
+    }
+
+    ox=x; oy=y;    
+   
+    state->buttons=                      /* for Ultra-Pad and graphire */
+      !!(data[3]&8)  * GPM_B_LEFT  +
+      !!(data[3]&16) * GPM_B_RIGHT +
+      !!(data[3]&32) * GPM_B_MIDDLE;   /* UD: rom-version >=1.3 */
+    return 0;
 }
 
-
 /* Calcomp UltraSlate tablet (John Anderson)
  * modeled after Wacom and NCR drivers (thanks, guys)
  * Note: I don't know how to get the tablet size from the tablet yet, so
@@ -920,9 +1141,157 @@ static int M_mtouch(Gpm_Event *state,  unsigned char *data)
   #undef DIF_TIME
 }
 
+/*
+ * This decoder is copied and adapted from the above mtouch.
+ * However, this one uses argv, which should be ported above as well.
+ * These variables are used for option passing
+ */
+static int gunze_avg = 9; /* the bigger the smoother */
+static int gunze_calib[4]; /* x0,y0 x1,y1 (measured at 1/8 and 7/8) */
+static int gunze_debounce = 100; /* milliseconds: ignore shorter taps */
+
+static int M_gunze(Gpm_Event *state,  unsigned char *data)
+{
+    /*
+     * This generates button-1 events, by now.
+     * Check README.gunze for additional information.
+     */
+    int x, y;
+    static int avgx, avgy;          /* average over time, for smooth feeling */
+    static int upx, upy;            /* keep track of last finger-up place */
+    static int released = 0, dragging = 0;
+    static struct timeval uptv, tv; /* time of last up, and down events */
+    int timediff;
+    
+    #define REAL_TO_XCELL(x) (x * win.ws_col / 0x3FFF)
+    #define REAL_TO_YCELL(y) (y * win.ws_row / 0x3FFF)
+    
+    #define GET_TIME(tv) (gettimeofday(&tv, (struct timezone *)NULL))
+    #define DIF_TIME(t1,t2) ((t2.tv_sec -t1.tv_sec) *1000+ \
+                           (t2.tv_usec-t1.tv_usec)/1000)
+
+    if (data[0] == 'R') {
+	/*
+	 * finger-up event: this is usually offset a few pixels,
+	 * so ignore this x and y values. And invalidate avg.
+	 */
+	upx = avgx;
+	upy = avgy;
+	GET_TIME(uptv); /* ready for the next finger-down */
+	state->buttons = 0;
+	released=1;
+	return 0;
+    }
+
+    if (sscanf(data+1, "%d,%d", &x, &y)!= 2) {
+	gpm_debug_log(LOG_NOTICE, "gunze: invalid packet \"%s\"\n", data);
+	return 10; /* eat one byte only, leave ten of them */
+    }
+    /*
+     * First of all, calibrate decoded data;
+     * the four numbers are 1/8X 1/8Y, 7/8X, 7/8Y
+     */
+    x = 128 + 768 * (x - gunze_calib[0])/(gunze_calib[2]-gunze_calib[0]);
+    y = 128 + 768 * (y - gunze_calib[1])/(gunze_calib[3]-gunze_calib[1]);
+
+    /* range is 0-1023, rescale it upwards (0-REALPOS_MAX) */
+    x = x * REALPOS_MAX / 1023;
+    y = REALPOS_MAX - y * REALPOS_MAX / 1023;
+
+    if (x<0) x = 0; if (x > REALPOS_MAX) x = REALPOS_MAX;
+    if (y<0) y = 0; if (y > REALPOS_MAX) y = REALPOS_MAX;
+
+    /*
+     * there are some issues wrt press events: since the device needs a
+     * non-negligible pressure to detect stuff, it sometimes reports
+     * quick up-down sequences, that actually are just the result of a
+     * little bouncing of the finger. Therefore, we should discard any
+     * release-press pair. This can be accomplished at press time.
+     */
+
+    if (released) { /* press event -- or just bounce*/
+	GET_TIME(tv);
+	timediff = DIF_TIME(uptv, tv);
+	released = 0;
+	if (timediff > gunze_debounce && timediff < opt_time) {
+	    /* count as button press placed at finger-up pixel */
+	    dragging = 1;
+	    state->buttons = GPM_B_LEFT;
+	    realposx = avgx = upx; state->x = REAL_TO_XCELL(realposx);
+	    realposy = avgy = upy; state->y = REAL_TO_YCELL(realposy);
+	    upx = (upx - x); /* upx-upy become offsets to use for this drag */
+	    upy = (upy - y);
+	    return 0;
+	}
+	if (timediff <= gunze_debounce) {
+	    /* just a bounce, invalidate offset, leave dragging alone */
+	    upx = upy = 0;
+	} else {
+	    /* else, count as a new motion event, reset avg */
+	    dragging = 0;
+	    realposx = avgx = x;
+	    realposy = avgy = y;
+	}
+    }
+
+    state->buttons = 0;
+    if (dragging) { /* a drag event: use position relative to press */
+	x += upx;
+	y += upy;
+	state->buttons = GPM_B_LEFT;
+    }
+
+    realposx = avgx = (gunze_avg * avgx + x)/(gunze_avg+1);
+    state->x = REAL_TO_XCELL(realposx);
+    realposy = avgy = (gunze_avg * avgy + y)/(gunze_avg+1);
+    state->y = REAL_TO_YCELL(realposy);
+
+    return 0;
+
+    #undef REAL_TO_XCELL
+    #undef REAL_TO_YCELL
+    #undef GET_TIME
+    #undef DIF_TIME
+}
+
+/*  Genius Wizardpad tablet  --  Matt Kimball (mkimball@xmission.com)  */
+static int wizardpad_width = -1;
+static int wizardpad_height = -1;
+static int M_wp(Gpm_Event *state,  unsigned char *data)
+{
+	int x, y, pressure;
+
+	x = ((data[4] & 0x1f) << 12) | ((data[3] & 0x3f) << 6) | (data[2] & 0x3f);
+	state->x = x * win.ws_col / (wizardpad_width * 40);
+	realposx = x * 16383 / (wizardpad_width * 40);
+
+	y = ((data[7] & 0x1f) << 12) | ((data[6] & 0x3f) << 6) | (data[5] & 0x3f);
+	state->y = win.ws_row - y * win.ws_row / (wizardpad_height * 40) - 1;
+	realposy = 16383 - y * 16383 / (wizardpad_height * 40) - 1;  
+
+	pressure = ((data[9] & 0x0f) << 4) | (data[8] & 0x0f);
+
+	state->buttons=
+		(pressure >= 0x20) * GPM_B_LEFT +
+		!!(data[1] & 0x02) * GPM_B_RIGHT +
+		/* the 0x08 bit seems to catch either of the extra buttons... */
+		!!(data[1] & 0x08) * GPM_B_MIDDLE;
+
+	return 0;
+}
+
 /*========================================================================*/
 /* Then, mice should be initialized */
 
+static Gpm_Type*
+I_empty(int fd, unsigned short flags,
+	 struct Gpm_Type *type, int argc, char **argv)
+{
+    if (check_no_argv(argc, argv)) return NULL;
+    return type;
+}
+
+
 static int
 setspeed(int fd, int old, int new, int needtowrite, unsigned short flags)
 {
@@ -940,22 +1309,24 @@ setspeed(int fd, int old, int new, int needtowrite, unsigned short flags)
 
   switch (old)
     {
-    case 9600:	tty.c_cflag = flags | B9600; break;
-    case 4800:	tty.c_cflag = flags | B4800; break;
-    case 2400:	tty.c_cflag = flags | B2400; break;
-    case 1200:
-    default:	tty.c_cflag = flags | B1200; break;
+    case 19200: tty.c_cflag = flags | B19200; break;
+    case  9600: tty.c_cflag = flags |  B9600; break;
+    case  4800: tty.c_cflag = flags |  B4800; break;
+    case  2400: tty.c_cflag = flags |  B2400; break;
+    case  1200:
+    default:    tty.c_cflag = flags | B1200; break;
     }
 
   tcsetattr(fd, TCSAFLUSH, &tty);
 
   switch (new)
     {
-    case 9600:	c = "*q";  tty.c_cflag = flags | B9600; break;
-    case 4800:	c = "*p";  tty.c_cflag = flags | B4800; break;
-    case 2400:	c = "*o";  tty.c_cflag = flags | B2400; break;
-    case 1200:
-    default:	c = "*n";  tty.c_cflag = flags | B1200; break;
+    case 19200: c = "*r";  tty.c_cflag = flags | B19200; break;
+    case  9600: c = "*q";  tty.c_cflag = flags |  B9600; break;
+    case  4800: c = "*p";  tty.c_cflag = flags |  B4800; break;
+    case  2400: c = "*o";  tty.c_cflag = flags |  B2400; break;
+    case  1200:
+    default:    c = "*n";  tty.c_cflag = flags | B1200; break;
     }
 
   if (needtowrite) write(fd, c, 2);
@@ -981,11 +1352,14 @@ static struct {
 };
 
 static Gpm_Type*
-I_serial(int fd, unsigned short flags, struct Gpm_Type *type)
+I_serial(int fd, unsigned short flags,
+	 struct Gpm_Type *type, int argc, char **argv)
 {
   int i; unsigned char c;
   fd_set set; struct timeval timeout={0,0}; /* used when not debugging */
 
+  /* accept "-o dtr", "-o rts" and "-o both" */
+  if (option_modem_lines(fd, argc, argv)) return NULL;
 
 #ifndef DEBUG
   /* flush any pending input (thanks, Miguel) */
@@ -1051,12 +1425,15 @@ I_serial(int fd, unsigned short flags, struct Gpm_Type *type)
 }
 
 static Gpm_Type*
-I_logi(int fd, unsigned short flags, struct Gpm_Type *type)
+I_logi(int fd, unsigned short flags,
+       struct Gpm_Type *type, int argc, char **argv)
 {
   int i;
   struct stat buf;
   int busmouse;
 
+  if (check_no_argv(argc, argv)) return NULL;
+
   /* is this a serial- or a bus- mouse? */
   if (fstat(fd,&buf)==-1) oops("fstat()");
   i=MAJOR(buf.st_rdev);
@@ -1082,21 +1459,143 @@ I_logi(int fd, unsigned short flags, struct Gpm_Type *type)
   return type;
 }
 
-static Gpm_Type *I_wacom(int fd, unsigned short flags, struct Gpm_Type *type)
+static Gpm_Type *I_wacom(int fd, unsigned short flags,
+                         struct Gpm_Type *type, int argc, char **argv)
 {
-  int i;
+/* wacom graphire tablet */
+#define UD_RESETBAUD     "\r$"      /* reset baud rate to default (wacom V) */
+                                    /* or switch to wacom IIs (wacomIV)     */ 
+#define UD_RESET         "#\r"      /* Reset tablet and enable WACOM IV     */
+#define UD_SENDCOORDS    "ST\r"     /* Start sending coordinates            */
+#define UD_FIRMID        "~#\r"     /* Request firmware ID string           */
+#define UD_COORD         "~C\r"     /* Request max coordinates              */
+#define UD_STOP          "\nSP\r"   /* stop sending coordinates             */
+
+   void reset_wacom()
+   { 
+     /* Init Wacom communication; this is modified from xf86Wacom.so module */
+     /* Set speed to 19200 */
+     setspeed (fd, 1200, 19200, 0, B19200|CS8|CREAD|CLOCAL|HUPCL);
+     /* Send Reset Baudrate Command */ 
+     write(fd, UD_RESETBAUD, strlen(UD_RESETBAUD));
+     usleep(250000);   
+     /* Send Reset Command */
+     write(fd, UD_RESET,     strlen(UD_RESET));
+     usleep(75000);
+     /* Set speed to 9600bps */
+     setspeed (fd, 1200, 9600, 0, B9600|CS8|CREAD|CLOCAL|HUPCL);
+     /* Send Reset Command */
+     write(fd, UD_RESET, strlen(UD_RESET));
+     usleep(250000);  
+     write(fd, UD_STOP, strlen(UD_STOP));
+     usleep(100000);
+   }
+
+  int wait_wacom()
+   {
+    /* 
+       Wait up to 200 ms for Data from Tablet. 
+       Do not read that data.
+       Give back 0 on timeout condition, -1 on error and 1 for DataPresent
+    */
+    struct timeval timeout;
+    fd_set readfds;
+    int err;
+    FD_ZERO(&readfds);  FD_SET(fd, &readfds);
+    timeout.tv_sec = 0; timeout.tv_usec = 200000;
+    err = select(FD_SETSIZE, &readfds, NULL, NULL, &timeout);
+    return((err>0)?1:err);
+   }
+
+  char buffer[50], *p;
+
+  int RequestData(char *cmd)
+  {
+    int err;
+    /* 
+       Send cmd if not null, and get back answer from tablet. 
+       Get Data to buffer until full or timeout.
+       Give back 0 for timeout and !0 for buffer full
+    */
+    if (cmd) write(fd,cmd,strlen(cmd));
+    memset(buffer,0,sizeof(buffer)); p=buffer;
+    err=wait_wacom();
+    while (err != -1 && err && (p-buffer)<(sizeof(buffer)-1))
+    {
+      p+= read(fd,p,(sizeof(buffer)-1)-(p-buffer));
+      err=wait_wacom();
+    }
+
+    /* return 1 for buffer full */
+    return ((strlen(buffer) >= (sizeof(buffer)-1))? !0 :0);
+    
+  }
+
+  /*
+    We do both modes, relative and absolute, with the same function.
+    If WacomAbsoluteWanted is !0 then that function calculates
+    absolute values, else relative ones.
+    We set this by the -o switch:
+    gpm -t wacom -o absolute   # does absolute mode
+    gpm -t wacom               # does relative mode
+    gpm -t wacom -o relative   # does relative mode
+  */
+
+  /* accept boolean options absolute and relative */
+  static argv_helper optioninfo[] = {
+         {"absolute",  ARGV_BOOL, u: {iptr: &WacomAbsoluteWanted}, value: !0},
+         {"relative",  ARGV_BOOL, u: {iptr: &WacomAbsoluteWanted}, value:  0},
+         {"",       ARGV_END} 
+  };
+  parse_argv(optioninfo, argc, argv); 
+  type->absolute = WacomAbsoluteWanted;
+  reset_wacom();
+
+  /* "Flush" input queque */
+  while (RequestData(NULL));
+
+  /* read WACOM-ID */
+  RequestData(UD_FIRMID); 
+
+  /* Search for matching modell */
+  for ( WacomModell=0; 
+        WacomModell< (sizeof(wcmodell) / sizeof(struct WC_MODELL)); 
+        WacomModell++ ) {
+      if (!strncmp(buffer+2,wcmodell[WacomModell].magic, 2))
+      {
+        /* Magic matches, modell found */
+        wmaxx=wcmodell[WacomModell].maxX;
+        wmaxy=wcmodell[WacomModell].maxY;
+        break;
+      }
+  }      
+  if (WacomModell >= (sizeof(wcmodell) / sizeof(struct WC_MODELL))) 
+     WacomModell=-1;
+  gpm_debug_log(LOG_NOTICE, "WACOM Mode: %c Modell: %s (Answer: %s)\n", 
+                type->absolute? 'A':'R',
+                (WacomModell==(-1))? "Unknown" : wcmodell[WacomModell].name,
+                buffer+2
+  );
+   
+  /* read Wacom max size */
+  if(WacomModell!=(-1) && (!wcmodell[WacomModell].maxX)){
+    RequestData(UD_COORD);
+    sscanf(buffer+2, "%d,%d", &wmaxx, &wmaxy);
+    wmaxx = (wmaxx-wcmodell[WacomModell].border);
+    wmaxy = (wmaxy-wcmodell[WacomModell].border);
+  }
+  write(fd,UD_SENDCOORDS,4);
 
-  if (opt_baud == 1200) opt_baud=9600; /* default to 9600 */
-  /* change from any available speed to the chosen one */
-  for (i=9600; i>=1200; i/=2)
-    setspeed(fd, i, opt_baud, 1 /* write */, flags);
   return type;
 }
 
-static Gpm_Type *I_pnp(int fd, unsigned short flags, struct Gpm_Type *type)
+static Gpm_Type *I_pnp(int fd, unsigned short flags,
+		       struct Gpm_Type *type, int argc, char **argv)
 {  
   struct termios tty;
-  unsigned char buffer[255];
+
+  /* accept "-o dtr", "-o rts" and "-o both" */
+  if (option_modem_lines(fd, argc, argv)) return NULL;
 
   /*
    * Just put the device to 1200 baud. Thanks to Francois Chastrette
@@ -1122,10 +1621,14 @@ static Gpm_Type *I_pnp(int fd, unsigned short flags, struct Gpm_Type *type)
 }
 
 /* intellimouse, ps2 version: Ben Pfaff and Colin Plumb */
-static Gpm_Type *I_imps2(int fd, unsigned short flags, struct Gpm_Type *type)
+static Gpm_Type *I_imps2(int fd, unsigned short flags,
+			 struct Gpm_Type *type, int argc, char **argv)
 {
   static unsigned char s1[] = { 243, 200, 243, 100, 243, 80, };
   static unsigned char s2[] = { 246, 230, 244, 243, 100, 232, 3, };
+
+  if (check_no_argv(argc, argv)) return NULL;
+
   write (fd, s1, sizeof (s1));
   usleep (30000);
   write (fd, s2, sizeof (s2));
@@ -1134,35 +1637,42 @@ static Gpm_Type *I_imps2(int fd, unsigned short flags, struct Gpm_Type *type)
   return type;
 }
 
-static Gpm_Type *I_twid(int fd, unsigned short flags, struct Gpm_Type *type)
+static Gpm_Type *I_twid(int fd, unsigned short flags,
+			struct Gpm_Type *type, int argc, char **argv)
 {
+
+  if (check_no_argv(argc, argv)) return NULL;
+
   if (twiddler_key_init() != 0) return NULL;
   /*
    * the twiddler is a serial mouse: just drop dtr
    * and run at 2400 (unless specified differently) 
    */
   if(opt_baud==DEF_BAUD) opt_baud = 2400;
-  opt_toggle = TIOCM_DTR;
-  return I_serial(fd, flags, type);
+  argv[1] = "dtr"; /* argv[1] is guaranteed to be NULL (this is dirty) */
+  return I_serial(fd, flags, type, argc, argv);
 }
 
-static Gpm_Type *I_calus(int fd, unsigned short flags, struct Gpm_Type *type) {
-   int i;
+static Gpm_Type *I_calus(int fd, unsigned short flags,
+			 struct Gpm_Type *type, int argc, char **argv)
+{
+   if (check_no_argv(argc, argv)) return NULL;
 
    if (opt_baud == 1200) opt_baud=9600; /* default to 9600 */
-
-   return I_serial(fd, flags, type);
+   return I_serial(fd, flags, type, argc, argv);
 }
 
 /* synaptics touchpad, ps2 version: Henry Davies */
-static Gpm_Type *I_synps2(int fd, unsigned short flags, struct Gpm_Type *type)
+static Gpm_Type *I_synps2(int fd, unsigned short flags,
+			  struct Gpm_Type *type, int argc, char **argv)
 {
   syn_ps2_init (fd);
   return type;
 }
 
 
-static Gpm_Type *I_summa(int fd, unsigned short flags, struct Gpm_Type *type) 
+static Gpm_Type *I_summa(int fd, unsigned short flags,
+			 struct Gpm_Type *type, int argc, char **argv) 
 {
    void resetsumma()
    {
@@ -1180,9 +1690,6 @@ static Gpm_Type *I_summa(int fd, unsigned short flags, struct Gpm_Type *type)
     return(err);
    }
   int err;
-  struct termios tty;
-  fd_set readfds;
-  struct timeval timeout;
   char buffer[255];
   char config[5];
   /* Summasketchtablet (from xf86summa.o: thanks to Steven Lang <tiger@tyger.org>)*/
@@ -1261,7 +1768,8 @@ static Gpm_Type *I_summa(int fd, unsigned short flags, struct Gpm_Type *type)
   return type;
 }
 
-static Gpm_Type *I_mtouch(int fd, unsigned short flags, struct Gpm_Type *type) 
+static Gpm_Type *I_mtouch(int fd, unsigned short flags,
+			  struct Gpm_Type *type, int argc, char **argv)
 {
   struct termios tty;
 
@@ -1283,6 +1791,127 @@ static Gpm_Type *I_mtouch(int fd, unsigned short flags, struct Gpm_Type *type)
   return type;
 }
 
+/* simple initialization for the gunze touchscreen */
+static Gpm_Type *I_gunze(int fd, unsigned short flags,
+			  struct Gpm_Type *type, int argc, char **argv)
+{
+    struct termios tty;
+    FILE *f;
+    char s[80];
+    int i, calibok = 0;
+
+    #define GUNZE_CALIBRATION_FILE SYSCONFDIR "/gpm-calibration"
+    /* accept a few options */
+    static argv_helper optioninfo[] = {
+	{"smooth",   ARGV_INT, u: {iptr: &gunze_avg}},
+	{"debounce", ARGV_INT, u: {iptr: &gunze_debounce}},
+	/* FIXME: add corner tapping */
+	{"",       ARGV_END}
+    };
+    parse_argv(optioninfo, argc, argv);
+    
+    /* check that the baud rate is valid */
+    if (opt_baud == DEF_BAUD) opt_baud = 19200; /* force 19200 as default */
+    if (opt_baud != 9600 && opt_baud != 19200) {
+	fprintf(stderr, "%s: %s: wrong baud rate, using 19200\n",
+		prgname, argv[0]);
+	opt_baud = 19200;
+    }
+    tcgetattr(fd, &tty);
+    tty.c_iflag = IGNBRK | IGNPAR;
+    tty.c_oflag = 0;
+    tty.c_lflag = 0;
+    tty.c_line = 0;
+    tty.c_cc[VTIME] = 0;
+    tty.c_cc[VMIN] = 1;
+    tty.c_cflag = (opt_baud == 9600 ? B9600 : B19200) |CS8|CREAD|CLOCAL|HUPCL;
+    tcsetattr(fd, TCSAFLUSH, &tty);
+
+    /* FIXME: try to find some information about the device */
+
+    /* retrieve calibration, if not existent, use defaults (uncalib) */
+    f = fopen(GUNZE_CALIBRATION_FILE, "r");
+    if (f) {
+	fgets(s, 80, f); /* discard the comment */
+	if (fscanf(f, "%d %d %d %d", gunze_calib, gunze_calib+1,
+		   gunze_calib+2, gunze_calib+3) == 4)
+	    calibok = 1;
+	/* Hmm... check */
+	for (i=0; i<4; i++)
+	    if (gunze_calib[i] & ~1023) calibok = 0;
+	if (gunze_calib[0] == gunze_calib[2]) calibok = 0;
+	if (gunze_calib[1] == gunze_calib[4]) calibok = 0;
+	fclose(f);
+	}
+    if (!calibok) {
+	fprintf(stderr,	"%s: gunze: calibration data absent or invalid, "
+		"using defaults\n", prgname);
+	gunze_calib[0] = gunze_calib[1] = 128; /* 1/8 */
+	gunze_calib[2] = gunze_calib[3] = 896; /* 7/8 */
+    }
+    return type;
+}
+
+/*  Genius Wizardpad tablet  --  Matt Kimball (mkimball@xmission.com)  */
+static Gpm_Type *I_wp(int fd, unsigned short flags,
+		      struct Gpm_Type *type, int argc, char **argv)
+{
+  struct termios tty;
+  char tablet_info[256];
+  int count, pos, size;
+
+  /* Set speed to 9600bps (copied from I_summa, above :) */
+  tcgetattr(fd, &tty);
+  tty.c_iflag = IGNBRK | IGNPAR;
+  tty.c_oflag = 0;
+  tty.c_lflag = 0;
+  tty.c_line = 0;
+  tty.c_cc[VTIME] = 0;
+  tty.c_cc[VMIN] = 1;
+  tty.c_cflag = B9600|CS8|CREAD|CLOCAL|HUPCL;
+  tcsetattr(fd, TCSAFLUSH, &tty);
+
+  /*  Reset the tablet (':') and put it in remote mode ('S') so that
+	  it isn't sending anything to us.  */
+  write(fd, ":S", 2);
+  tcsetattr(fd, TCSAFLUSH, &tty);
+
+  /*  Query the model of the tablet  */
+  write(fd, "T", 1);
+  sleep(1);
+  count = read(fd, tablet_info, 255);
+
+  /*  The tablet information should start with "KW" followed by the rest of
+	  the model number.  If it isn't there, it probably isn't a WizardPad.  */
+  if(count < 2)
+	  return NULL;
+  if(tablet_info[0] != 'K' || tablet_info[1] != 'W')
+	  return NULL;
+
+  /*  Now, we want the width and height of the tablet.  They should be 
+	  of the form "X###" and "Y###" where ### is the number of units of
+	  the tablet.  The model I've got is "X130" and "Y095", but I guess
+	  there might be other ones sometime.  */
+  for(pos = 0; pos < count; pos++) {
+	  if(tablet_info[pos] == 'X' || tablet_info[pos] == 'Y') {
+		  if(pos + 3 < count) {
+			  size = (tablet_info[pos + 1] - '0') * 100 +
+				     (tablet_info[pos + 2] - '0') * 10 +
+				     (tablet_info[pos + 3] - '0');
+			  if(tablet_info[pos] == 'X') {
+				  wizardpad_width = size;
+			  } else {
+				  wizardpad_height = size;
+			  }
+		  }
+	  }
+  }
+
+  /*  Set the tablet to stream mode with 180 updates per sec.  ('O')  */
+  write(fd, "O", 1);
+
+  return type;
+}
 
 /*========================================================================*/
 /* Finally, the table */
@@ -1342,17 +1971,17 @@ Gpm_Type mice[]={
            "Logitech", M_logi, I_logi, CS8 | CSTOPB | STD_FLG,
                                 {0xe0, 0x80, 0x80, 0x00}, 3, 3, 0, 0, 0},
   {"bm",   "Micro$oft busmice and compatible devices.",
-           "BusMouse", M_bm, NULL, STD_FLG, /* bm is sun */
+           "BusMouse", M_bm, I_empty, STD_FLG, /* bm is sun */
                                 {0xf8, 0x80, 0x00, 0x00}, 3, 3, 0, 0, 0},
   {"ps2",  "Busmice of the ps/2 series. Most busmice, actually.",
-           "PS/2", M_ps2, NULL, STD_FLG,
+           "PS/2", M_ps2, I_empty, STD_FLG,
                                 {0xc0, 0x00, 0x00, 0x00}, 3, 1, 0, 0, 0},
   {"ncr",  "Ncr3125pen, found on some laptops",
            "", M_ncr, NULL, STD_FLG,
                                 {0x08, 0x08, 0x00, 0x00}, 7, 7, 0, 1, 0},
-  {"wacom",  "Wacom tablet",
+  {"wacom","Wacom Protocol IV Tablets: Pen+Mouse, relative+absolute mode",
            "", M_wacom, I_wacom, STD_FLG,
-                                {0x80, 0x80, 0x80, 0x00}, 7, 7, 0, 0, 0},
+                                {0x80, 0x80, 0x80, 0x00}, 7, 1, 0, 0, 0},
   {"genitizer", "\"Genitizer\" tablet, in relative mode.",
            "", M_geni, I_serial, CS8|PARENB|PARODD,
                                 {0x80, 0x80, 0x00, 0x00}, 3, 1, 0, 0, 0},
@@ -1385,7 +2014,7 @@ Gpm_Type mice[]={
                                 {0x40, 0x40, 0x40, 0x00}, 6, 6, 1, 0, 0},
   {"synps2", "The \"Synaptics\" PS/2 TouchPad",
            "synaptics_ps2", M_synaptics_ps2, I_synps2, STD_FLG,
-                                {0xc8, 0x80, 0x00, 0x00}, 6, 1, 1, 0, 0},
+                                {0x80, 0x80, 0x00, 0x00}, 6, 1, 1, 0, 0},
   {"brw",  "Fellowes Browser - 4 buttons (and a wheel) (dual protocol?)",
            "", M_brw, I_pnp, CS7 | STD_FLG,
                                 {0xc0, 0x40, 0xc0, 0x00}, 4, 1, 0, 0, 0},
@@ -1398,15 +2027,22 @@ Gpm_Type mice[]={
 
   {"summa",  "Summagraphics or Genius tablet absolute mode(MM-Series)",
            "", M_summa, I_summa, STD_FLG,
-                                {0x98, 0x98, 0x00, 0x00}, 5, 1, 0, 1, R_summa}, 
+                                {0x80, 0x80, 0x00, 0x00}, 5, 1, 0, 1, R_summa},
   {"mtouch",  "MicroTouch touch-screens (only button-1 events, by now)",
            "", M_mtouch, I_mtouch, STD_FLG,
                                 {0x80, 0x80, 0x80, 0x00}, 5, 1, 0, 1, NULL}, 
+  {"gunze",  "Gunze touch-screens (only button-1 events, by now)",
+           "", M_gunze, I_gunze, STD_FLG,
+                                {0xF9, 0x50, 0xF0, 0x30}, 11, 1, 0, 1, NULL}, 
 
   {"acecad",  "Acecad tablet absolute mode(Sumagrapics MM-Series mode)",
            "", M_summa, I_summa, STD_FLG,
                                 {0x80, 0x80, 0x00, 0x00}, 7, 1, 0, 1, 0}, 
 
+  {"wp",      "Genius WizardPad tablet",
+           "wizardpad", M_wp, I_wp, STD_FLG,
+                                {0xFA, 0x42, 0x00, 0x00}, 10, 1, 0, 1, 0},
+
   {"",     "",
            "", NULL, NULL, 0,
                                 {0x00, 0x00, 0x00, 0x00}, 0, 0, 0, 0, 0}
@@ -1421,16 +2057,23 @@ int M_listTypes(void)
 
   printf("\n" GPM_NAME " " GPM_RELEASE ", " GPM_DATE "\n");
   printf("Available mouse types are:\n\n");
-  printf("  name   synonym         description\n\n");
+  printf("r name   synonym         description\n\n");
   for (type=mice; type->fun; type++)
-    printf("  %-6s %-12s %s\n",type->name, type->syn, type->desc);
+    printf("%c %-8s %s\n            Synonyms: %s\n", type->repeat_fun ?'*':' ',
+	   type->name, type->desc, type->synonyms);
 
   putchar('\n');
 
   return 1; /* to exit() */
 }
 
+
 
+/*
+ * Use 4 as indent-level. It used to be 2, but new stuff will be inserted
+ * using 4.
+ */
+
 /* Local Variables: */
-/* c-indent-level: 2 */
+/* c-indent-level: 4 */
 /* End: */
