@@ -22,10 +22,10 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  ********/
 
-static char rcsid[] = "$Id: mouse-test.c,v 1.5 1999/03/31 08:03:33 itz Exp $";
+static char rcsid[] = "$Id: mouse-test.c,v 1.3 2000/01/17 22:23:03 rubini Exp $";
 
 /* This code is horrible. Browse it at your risk */
 
@@ -54,7 +54,8 @@ static char rcsid[] = "$Id: mouse-test.c,v 1.5 1999/03/31 08:03:33 itz Exp $";
 struct mouse_features mymouse = {
   DEF_TYPE, DEF_DEV, DEF_SEQUENCE,
   DEF_BAUD, DEF_SAMPLE, DEF_DELTA, DEF_ACCEL, DEF_SCALE, DEF_SCALE /*scaley*/,
-  DEF_TIME, DEF_CLUSTER, DEF_THREE, DEF_TOGGLE, DEF_GLIDEPOINT_TAP,
+  DEF_TIME, DEF_CLUSTER, DEF_THREE, DEF_GLIDEPOINT_TAP,
+  (char *)NULL /* extra */,
   (Gpm_Type *)NULL
 };
 
@@ -154,7 +155,8 @@ struct device **gpm_makedev(struct device **current, char *name)
   return &((*current)->next);
 }
 
-Gpm_Type *(*I_serial)(int fd, unsigned short flags, struct Gpm_Type *type);
+Gpm_Type *(*I_serial)(int fd, unsigned short flags, struct Gpm_Type *type,
+		      int argc, char **argv);
 
 
 int mousereopen(int oldfd, char *name, Gpm_Type *type)
@@ -164,7 +166,7 @@ int fd;
   close(oldfd); usleep(100000);
   fd=open(name,O_RDWR);
   if (fd<0) oops(name);
-  (*I_serial)(fd,type->flags,type); /* ms initialization */
+  (*I_serial)(fd,type->flags,type,1,&type->name); /* ms initialization */
   return fd;
 }
 
@@ -330,7 +332,8 @@ int main(int argc, char **argv)
       printf("\t%s\r\n", (*nextdev)->name);
       FD_SET((*nextdev)->fd,&devSet);
       maxfd=max((*nextdev)->fd,maxfd);
-      (*I_serial)((*nextdev)->fd,(mice+1)->flags,mice+1); /* try ms mode */
+      (*I_serial)((*nextdev)->fd,(mice+1)->flags,mice+1,
+		  1, &(mice+1)->name); /* try ms mode */
       }
 
     savSet=devSet;
@@ -407,8 +410,8 @@ int main(int argc, char **argv)
     if (!*nextitem) oops("malloc()");
     (*nextitem)->this=cursor; (*nextitem)->next=NULL;
     printf("\t%s",cursor->name);
-    if (cursor->syn[0])
-      printf(" (%s)",cursor->syn);
+    if (cursor->synonyms && cursor->synonyms[0])
+      printf(" (%s)",cursor->synonyms);
     printf("\r\n");
     typecount++;
     nextitem=&((*nextitem)->next);
