@@ -1,7 +1,9 @@
 /*
  * synaptics.c - support for the synaptics serial touchpads
  *
- * Copyright 1999   hdavies@ameritech.net (Henry Davies
+ * Copyright 1999   hdavies@ameritech.net (Henry Davies)
+ *                  Geert Van der Plas provided the code to support
+ *                        older Synaptics PS/2 touchpads.
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -15,9 +17,8 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  *
- * The PS/2 initialization code
  ********/
 
 /*
@@ -70,6 +71,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <string.h>
 #include <sys/time.h>
 
@@ -697,10 +699,10 @@ void syn_read_config_file ()
   int param;
   FILE *config;
 
-  if (config = fopen (SYSCONFDIR "/gpm-syn.conf", "r")) {
+  if ( (config = fopen (SYSCONFDIR "/gpm-syn.conf", "r")) ) {
     while (fgets (line, 80, config)) {
       if (line [0] == '[') {
-	if (token = strtok (line, "[] ")) {
+	if ( (token = strtok (line, "[] ")) ) {
 	  param = 0;
 
 	  /* which param is it */
@@ -887,12 +889,27 @@ static void ser_read (int fd,
 		      unsigned char *bytes,
 		      int count) 
 {
+  struct timeval tv;
+  fd_set rfds;
   int num_read = 0;
   int read_count;
 
-  while (num_read < count) {
+  FD_ZERO (&rfds);
+  FD_SET (fd, &rfds);
+  tv.tv_sec = 1;
+  tv.tv_usec = 0;
+
+  while ((select (fd+1, &rfds, NULL, NULL, &tv) == 1) &&
+	 (num_read < count)) {
     read_count = read (fd, &bytes [num_read], count - num_read);
     num_read += read_count;
+
+    FD_ZERO (&rfds);
+    FD_SET (fd, &rfds);
+  }
+
+  for (; num_read < count; num_read++) {
+     bytes [num_read] = '\0';
   }
 }
 
@@ -1050,7 +1067,6 @@ static void ps2_status_rqst(int fd,
 			    byte cmd,
 			    byte *bytes) 
 {
-  byte chk;
   ps2_send_cmd(fd, cmd);
   ps2_putbyte(fd, 0xE9);
   bytes [0]=ps2_getbyte(fd);
@@ -1122,6 +1138,21 @@ static void syn_translate_ps2_report (unsigned char *data,
 			((data [3] & 0x20) << 7) |
 			data [5]);
     report->pressure = data [2];
+  } else if (((data [0] & 0xc0) == 0xc0) &&
+	     ((data [1] & 0x60) == 0x00) &&
+	     ((data [3] & 0xc0) == 0x80) &&
+	     ((data [4] & 0x60) == 0x00)) {
+    report->gesture  = check_bits (data [0], 0x04);
+    report->finger   = check_bits (data [1], 0x80);
+    report->left     = check_bits (data [0], 0x01);
+    report->middle   = 0;
+    report->right    = check_bits (data [0], 0x02);
+    report->x        = (((data [1] & 0x1f) << 8) |
+			data [2]);
+    report->y        = (((data [4] & 0x1f) << 8) |
+			data [5]);
+    report->pressure = (((data [0] & 0x30) << 2 ) |
+			(data [3] & 0x3f));
   } else {
     gpm_debug_log (LOG_NOTICE,"tossing PS/2 data: ");
     for (i = 0; i < 6; i++)
