@@ -2,6 +2,7 @@
  * wd.h - some stupid things to enable messages
  *
  * Copyright 1994   DIS - Universita` di Pavia - Italy
+ * Copyright (C) 1998 Ian Zimmerman <itz@rahul.net>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -15,38 +16,37 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  ********/
 
 #ifndef _WD_H_
 #define _WD_H_
 
-#include <stdio.h>
+/* $Id: wd.h,v 1.2 2000/01/17 22:23:03 rubini Exp $ */
 
-/*========== the WD macro prints on stderr to help tracing program flow =*/
-
-#if defined DEBUG
-#  ifdef TCL_MEM_DEBUG
-#  define WD (fprintf(stderr,"  File " __FILE__ "(%i)\n", __LINE__),\
-               Tcl_ValidateAllMemory(__FILE__,__LINE__))
-#  else
-#  define WD fprintf(stderr,"File " __FILE__ "(%i)\n", __LINE__)
-#  endif
+#ifdef HAVE_SYSLOG_H
+#include <syslog.h>
 #else
-#define WD 
+#define LOG_EMERG 0
+#define LOG_ALERT 1
+#define LOG_CRIT 2
+#define LOG_ERR 3
+#define LOG_WARNING 4
+#define LOG_NOTICE 5
+#define LOG_INFO 6
+#define LOG_DEBUG 7
 #endif
 
+extern int 
+gpm_debug_level;
 
-/*========== the PDEBUG symbol masks fprintf =======*/
+extern int 
+gpm_log_daemon;
 
-#ifdef DEBUG
-#define PDEBUG(data) fprintf data
-#else
-#define PDEBUG(data)
-#endif
+extern void
+gpm_debug_log(int level, char* fmt, ...);
 
-/*========== the PDEBUGG symbol hides everything =======*/
-
-#define PDEBUGG(data)
+extern void 
+gpm_oops(char *f, int n, char *s, ...);
 
 #endif /* _WD_H_ */
